﻿using System;
using System.Text;
using System.Net;
using System.Net.Sockets;

using RemotingInterfaces;

namespace GameServer
{
    // This is the class that listens for messages from the proxy
    // and passes them on to the real object.
    // Socket code inspired on http://www.developerfusion.co.uk/show/3918/
    public class LobbySkeleton
    {
        private ILobby Lobby { get; set; }
        private Socket ListenSocket { get; set; }
        private Socket WorkerSocket { get; set; }
        private AsyncCallback WorkerCallBack { get; set; }

        public LobbySkeleton(ILobby lobby)
        {
            Lobby = lobby;
            WorkerCallBack = new AsyncCallback(IncomingData);
        }

        public void Listen()
        {
            // Start listening on a socket on port 8081
            ListenSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            IPEndPoint listenPoint = new IPEndPoint(IPAddress.Any, 8081);
            ListenSocket.Bind(listenPoint);
            ListenSocket.Listen(10);
            ListenSocket.BeginAccept(new AsyncCallback(AcceptUser), null);
        }

        public void Stop()
        {
            // Stop the listening socket
            ListenSocket.Close();
        }

        public void AcceptUser(IAsyncResult asyncResult)
        {
            try
            {
                WorkerSocket = ListenSocket.EndAccept(asyncResult);
                WaitForData(WorkerSocket);
            }
            catch (SocketException)
            {
                // Socket has been closed, don't really care for this example
            }
        }

        private void WaitForData(Socket workerSocket)
        {
            if (WorkerCallBack == null)
            {
                WorkerCallBack = new AsyncCallback(IncomingData);
            }

            Packet incomingPacket = new Packet(workerSocket);
            workerSocket.BeginReceive(incomingPacket.DataBuffer, 0, incomingPacket.DataBuffer.Length, SocketFlags.None, IncomingData, incomingPacket);
        }

        public void IncomingData(IAsyncResult asyncResult)
        {
            try
            {
                Packet incomingPacket = (Packet)asyncResult.AsyncState;
                int iRx = incomingPacket.WorkerSocket.EndReceive(asyncResult);

                char[] chars = new char[iRx + 1];
                Decoder d = Encoding.UTF8.GetDecoder();
                int charLen = d.GetChars(incomingPacket.DataBuffer, 0, iRx, chars, 0);

                String inputCommand = new String(chars);
                Console.WriteLine(inputCommand);

                string[] inputPieces = inputCommand.Split('$');
                string callMethod = inputPieces[0];
                string callParams = inputPieces[1];

                byte[] sendData = Encoding.ASCII.GetBytes("");
                switch (callMethod)
                {
                    case "TotalUsers": 
                        sendData = Encoding.ASCII.GetBytes(Lobby.TotalUsers.ToString());
                        break;
                    case "Users":
                        sendData = Encoding.ASCII.GetBytes(string.Join("|", Lobby.Users.ToArray()));
                        break;
                    case "RegisterUser": 
                        Lobby.RegisterUser(callParams);
                        sendData = Encoding.ASCII.GetBytes("OK");
                        break;
                }
                WorkerSocket.Send(sendData);

                WaitForData(WorkerSocket);
            }
            catch (SocketException)
            {
                // Socket has been closed, don't really care for this example
            }
        }

        private class Packet
        {
            public Socket WorkerSocket { get; set; }
            public byte[] DataBuffer { get; set; }

            public Packet(Socket workerSocket)
            {
                WorkerSocket = workerSocket;
                DataBuffer = new byte[100];
            }
        }
    }
}
